@@ -10,7 +10,7 @@ from collections import defaultdict
 import itertools
 import math
 
-def Die(arg, min_outcome=None, ndim=None):
+def Die(*args, min_outcome=None, ndim=None):
     """ Factory for constructing a die.
     
     This is capitalized because it is the preferred way of getting a new instance,
@@ -22,22 +22,23 @@ def Die(arg, min_outcome=None, ndim=None):
     * `hdroller.Die(6)`: A die that always rolls the `int` 6.
     * `hdroller.d(6)`: A d6.
     
-    Here are different ways to create a d6:
-    
-    * Provide the die itself: `hdroller.Die(hdroller.d6)`
-    * Provide a mapping from outcomes to weights: `hdroller.Die({1:1, 2:1, 3:1, 4:1, 5:1, 6:1})`
-    * Provide a sequence of outcomes: `hdroller.Die([1, 2, 3, 4, 5, 6])`
-    * Provide a consecutive sequence of weights with a minimum outcome: `hdroller.Die([1, 1, 1, 1, 1, 1], min_outcome=1)`
-    
     Args:
-        arg: This can be one of the following, with examples of how to create a d6:
-            * A die, which will be returned itself.
-            * A mapping from outcomes to weights.
-            * A sequence of outcomes. Each outcome will be given weight `1` per time it appears.
-            * A sequence of weights, with `min_outcome` set to an `int`. 
+        *args: This can be one of the following, with examples of how to create a d6 where applicable:
+            * A single argument die, which will be returned itself.
+                `d6 == Die(d6)`.
+            * A single argument dict-like that maps outcomes to weights.
+                This option will be taken in preference to treating the dict-like itself as an outcome
+                even if the dict-like itself is hashable and comparable.
+                `d6 == Die({1:1, 2:1, 3:1, 4:1, 5:1, 6:1})`.
+            * A single argument sequence of weights, with `min_outcome` set to an `int`. 
                 The outcomes will be `int`s starting at `min_outcome`.
-            * A single hashable and comparable value.
-                There will be a single outcome equal to the argument, with weight `1`.
+                `d6 == Die([1, 1, 1, 1, 1, 1], min_outcome=1)`.
+            * Zero or more arguments, each one denoting an outcome with weight 1 per appearance.
+                The outcomes must be hashable and comparable.
+                `d6 == Die(1, 2, 3, 4, 5, 6)`.
+                Note that a single argument sequence will be treated as a single outcome; 
+                for example, `Die((1, 2, 3, 4, 5, 6))` is a die that always rolls the tuple `(1, 2, 3, 4, 5, 6)`.
+        min_outcome: Only used with a sequence of weights, as above.
         ndim: If set to `'scalar'`, the die will be forced to be scalar.
             If set to an `int`, the die will be forced to be vector with that number of dimensions.
             If omitted, this will be automatically detected.
@@ -46,8 +47,9 @@ def Die(arg, min_outcome=None, ndim=None):
     
     Raises:
         `ValueError` if `ndim` is set but is not consistent with `arg`.
+            Also, `None` is not a valid outcome for a die.
     """
-    data = _make_data(arg, min_outcome)
+    data = _make_data(args, min_outcome)
         
     ndim = _calc_ndim(data, ndim)
     
@@ -57,26 +59,25 @@ def Die(arg, min_outcome=None, ndim=None):
     else:
         return hdroller.die.scalar.ScalarDie(data)
 
-def _make_data(arg, min_outcome=None):
+def _make_data(args, min_outcome=None):
     """ Creates a `Weights` from the arguments. """
-    if isinstance(arg, hdroller.die.base.BaseDie):
-        data = arg._data
-    elif isinstance(arg, Weights):
-        data = arg
-    elif min_outcome is not None:
-        data = { min_outcome + i : weight for i, weight in enumerate(arg) }
-    elif hasattr(arg, 'keys') and hasattr(arg, '__getitem__'):
-        data = { k : arg[k] for k in arg.keys() }
-    elif hasattr(arg, '__iter__'):
-        data = defaultdict(int)
-        for v in arg:
-            data[v] += 1
-    else:
-        # Treat arg as the only possible value.
-        data = { arg : 1 }
+    if len(args) == 1:
+        arg = args[0]
+        if isinstance(arg, hdroller.die.base.BaseDie):
+            return arg._data
+        elif isinstance(arg, Weights):
+            return arg
+        elif min_outcome is not None:
+            return Weights({ min_outcome + i : weight for i, weight in enumerate(arg) })
+        elif hasattr(arg, 'keys') and hasattr(arg, '__getitem__'):
+            return Weights({ k : arg[k] for k in arg.keys() })
+
+    # Default case: each argument becomes an outcome.
+    data = defaultdict(int)
+    for outcome in args:
+        data[outcome] += 1
     
     return Weights(data)
-    
 
 def _calc_ndim(data, ndim):
     """Verifies `ndim` if provided and calculates it otherwise.
@@ -211,11 +212,13 @@ def mix(*dice, mix_weights=None, ndim=None, total_weight_method='lcm'):
     """ Constructs a die from a mixture of the input dice.
     
     This is equivalent to rolling a die and then choosing one of the input dice
-    based on the resulting outcome rolled. See `if_else()` for a simple example.
-    Also see `BaseDie.sub()`.
+    based on the resulting outcome rolled. See also `BaseDie.sub()`.
     
     Args:
         *dice: The dice to mix.
+            These can be values castable to dice.
+            Alternatively, values may be `hdroller.Reroll`,
+            in which case that value and the corresponding mix weight is simply dropped.
         mix_weights: An iterable of one `int` weight per input die.
             If not provided, all dice are mixed uniformly.
         total_weight_method: How to determine the total weight of the result.
@@ -228,48 +231,36 @@ def mix(*dice, mix_weights=None, ndim=None, total_weight_method='lcm'):
                 This is like rolling the above, but the specific mixing weight rolled
                 is used to help determine the result of the selected die.
     """
-    dice = align(*dice, ndim=ndim)
-    ndim = check_ndim(*dice)
-    
     if mix_weights is None:
         mix_weights = (1,) * len(dice)
+    
+    dice, mix_weights = zip(*((die, weight) for die, weight in zip(dice, mix_weights) if die is not hdroller.Reroll))
+    
+    dice = [Die(die, ndim=ndim) for die in dice]
+    ndim = check_ndim(*dice)
         
     if total_weight_method == 'prod':
-        weight_product = math.prod(d.total_weight() for d in dice)
+        weight_product = math.prod(die.total_weight() for die in dice)
     elif total_weight_method == 'lcm':
-        weight_product = math.lcm(*(d.total_weight() for d in dice))
+        weight_product = math.lcm(*(die.total_weight() for die in dice))
     elif total_weight_method == 'lcm_weighted':
-        weight_product = math.lcm(*(d.total_weight() * w for d, w in zip(dice, mix_weights)))
+        weight_product = math.lcm(*(die.total_weight() * w for die, w in zip(dice, mix_weights)))
     else:
         raise ValueError(f'Invalid total_weight_method {total_weight_method}.')
     
     data = defaultdict(int)
-    for d, mix_weight in zip(dice, mix_weights):
-        factor = mix_weight * weight_product // d.total_weight()
-        for outcome, weight in zip(d.outcomes(), d.weights()):
+    for die, mix_weight in zip(dice, mix_weights):
+        factor = mix_weight * weight_product // die.total_weight()
+        for outcome, weight in zip(die.outcomes(), die.weights()):
             data[outcome] += weight * factor
     return Die(data, ndim=ndim)
 
-def if_else(true_die, cond_die, false_die, total_weight_method='lcm'):
-    """ Roll `true_die` if `cond_die` else roll `false_die`.
-    
-    Also known as the ternary conditional operator.
-    
-    Args:
-        true_die: The die to roll if `cond_die.bool()` rolls `True`.
-        cond_die: The result of this die selects between `true_die` and `false_die`.
-        false_die: The die to roll if `cond_die.bool()` rolls `False`.
-        total_weight_method: As `hdroller.mix()`.
-    """
-    cond_die = cond_die.bool()
-    mix_weights = cond_die.weight_eq(True), cond_die.weight_eq(False)
-    return mix(true_die, false_die, mix_weights=mix_weights, total_weight_method=total_weight_method)
-
 def align(*dice, ndim=None):
     """Pads all the dice with zero weights so that all have the same set of outcomes.
     
     Args:
         *dice: Multiple dice or a single iterable of dice.
+        ndim: The number of dimensions of the result.
     
     Returns:
         A tuple of aligned dice.
@@ -277,14 +268,14 @@ def align(*dice, ndim=None):
     Raises:
         `ValueError` if the dice are of mixed ndims.
     """
-    dice = [Die(d, ndim=ndim) for d in dice]
+    dice = [Die(die, ndim=ndim) for die in dice]
     check_ndim(*dice)
-    union_outcomes = set(itertools.chain.from_iterable(die.outcomes() for die in dice))
-    return tuple(die.set_outcomes(union_outcomes) for die in dice)
+    outcomes = set(itertools.chain.from_iterable(die.outcomes() for die in dice))
+    return tuple(die.set_outcomes(outcomes) for die in dice)
 
 def align_range(*dice, ndim=None):
     """Pads all the dice with zero weights so that all have the same set of consecutive `int` outcomes. """
-    dice = [Die(d, ndim=ndim) for d in dice]
+    dice = [Die(die, ndim=ndim) for die in dice]
     check_ndim(*dice)
     outcomes = tuple(range(hdroller.min_outcome(*dice), hdroller.max_outcome(*dice) + 1))
     return tuple(die.set_outcomes(outcomes) for die in dice)
@@ -329,9 +320,9 @@ def apply(func, *dice, ndim=None):
     Returns:
         A die constructed from the outputs of `func` and the product of the weights of the dice.
     """
-    dice = [Die(d, ndim=ndim) for d in dice]
+    dice = [Die(die, ndim=ndim) for die in dice]
     data = defaultdict(int)
-    for t in itertools.product(*(d.items() for d in dice)):
+    for t in itertools.product(*(die.items() for die in dice)):
         outcomes, weights = zip(*t)
         data[func(*outcomes)] += math.prod(weights)
     
