@@ -4,7 +4,6 @@ import hdroller
 
 from abc import ABC, abstractmethod
 from collections import defaultdict
-from enum import Enum
 from functools import cached_property
 import itertools
 import math
@@ -44,14 +43,17 @@ class EvalPool(ABC):
     def next_state(self, state, outcome, *counts):
         """ State transition function.
         
-        This should produce a state given the previous state, an outcome, and the number of dice in each pool rolling that outcome.
+        This should produce a state given the previous state, an outcome,
+        and the number of dice in each pool rolling that outcome.
         
         Make sure to handle the base case where `state is None`.
         
         Args:
             state: A hashable object indicating the state before rolling the current outcome.
-                For the first call this will be `None`.
+                If there was no previous outcome, this will be `None`.
             outcome: The current outcome.
+                `next_state` will see all outcomes in consecutive order,
+                either ascending or descending depending on `direction()`.
             counts: One `int` for each pool indicating how many dice in that pool rolled the current outcome.
                 If there are multiple pools, it's possible that some outcomes will not appear in all pools.
                 In this case, the count for the pool(s) that do not have the outcome will be `None`. 
@@ -59,14 +61,17 @@ class EvalPool(ABC):
         
         Returns:
             A hashable object indicating the next state.
-            Alternatively, a value of `None` will drop the state from consideration,
-            effectively performing a full reroll.
+            The special value `hdroller.Reroll` can be used to immediately remove the state from consideration,
+            effectively performing a full reroll of the pool.
         """
     
     def final_outcome(self, final_state, *pools):
         """ Optional function to generate a final outcome from a final state.
         
         By default, the final outcome is equal to the final state.
+        Note that `None` is not a valid outcome for a die,
+        and if all pools consist of empty dice, the final state will be `None`.
+        Subclasses that want to handle this case should explicitly define what happens.
         
         Args:
             final_state: A state after all outcomes have been processed.
@@ -75,7 +80,7 @@ class EvalPool(ABC):
         Returns:
             A final outcome that will be used as part of constructing a die.
             This should be hashable and comparable.
-            Alternatively, a return value of `None` will drop the state from consideration,
+            Alternatively, a return value of `hdroller.Reroll` will drop the state from consideration,
             effectively performing a full reroll.
         """
         return final_state
@@ -86,41 +91,29 @@ class EvalPool(ABC):
         Note that an ascending (> 0) direction is not compatible with pools with `min_outcomes`,
         and a descending (< 0) direction is not compatible with pools with `max_outcomes`.
 
-        The default implementation chooses a direction automatically.
-        If only one direction is valid, it returns +2 / -2.
+        The default is ascending order.
         
         Args:
             *pools: One or more `DicePool`s being evaluated.
+            
+        Returns:
+            * > 0 if `next_state()` should always see the outcomes in ascending order.
+            * < 0 if `next_state()` should always see the outcomes in descending order.
+            * 0 if the order may be determined automatically.
         """
-        has_max_outcomes = any(pool.max_outcomes() is not None for pool in pools)
-        has_min_outcomes = any(pool.min_outcomes() is not None for pool in pools)
-        if has_max_outcomes and has_min_outcomes:
-            raise ValueError('Pools cannot be evaluated if they have both max_outcomes and min_outcomes.')
-        
-        if has_max_outcomes:
-            return 2
-        elif has_min_outcomes:
-            return -2
-        else:
-            num_drop_lowest = max(pool.num_drop_lowest() for pool in pools)
-            num_drop_highest = max(pool.num_drop_highest() for pool in pools)
-            return 1 if num_drop_lowest >= num_drop_highest else -1
+        return 1
     
     def ndim(self, *pools):
         """ Optional function to specify the number of dimensions of the output die.
         
-        The priority to determine ndim is as follows.
-        
-        1. The `ndim` argument to `eval()`, if not `None`.
-        2. This method, if not `None`.
-        3. Automatically determined by `Die()`.
+        If not provided, the ndim of the result will be determined automatically as per `Die()`.
         
         Args:
             *pools: One or more `DicePool`s being evaluated.
         
         Returns:
             The number of dimensions that the output die should have,
-            or `None` if this should be determined automatically by `die()`.
+            or `None` if this should be determined automatically by `Die()`.
         """
         return None
     
@@ -129,7 +122,7 @@ class EvalPool(ABC):
         """ A cache of (direction, pools) -> weight distribution over states. """
         return {}
     
-    def eval(self, *pools, ndim=None):
+    def eval(self, *pools):
         """ Evaluates pools.
         
         You can call the `EvalPool` object directly for the same effect,
@@ -140,30 +133,21 @@ class EvalPool(ABC):
                 Most evaluators will expect a fixed number of pools.
                 The outcomes of the pools must be mutually comparable.
                 Pools with `max_outcomes` and pools with `min_outcomes` are not compatible.
-            ndim: The number of dimensions of the resulting die.
-                If omitted, this will be determined automatically.
         
         Returns:
             A die representing the distribution of the final score.
         """
-        direction = self.direction(*pools)
+        algorithm, direction = self._select_algorithm(*pools)
         
-        dist = self._eval_internal(direction, *pools)
-        
-        if dist is None:
-            # All dice were empty.
-            return hdroller.Die(ndim=ndim)
+        dist = algorithm(direction, *pools)
         
         final_dist = defaultdict(int)
         for state, weight in dist.items():
             outcome = self.final_outcome(state, *pools)
-            if outcome is not None:
+            if outcome is not hdroller.Reroll:
                 final_dist[outcome] += weight
         
-        if ndim is None:
-            ndim = self.ndim(*pools)
-        
-        return hdroller.Die(final_dist, ndim=ndim)
+        return hdroller.Die(final_dist, ndim=self.ndim(*pools))
     
     __call__ = eval
     
@@ -187,15 +171,51 @@ class EvalPool(ABC):
         
         return bound_eval
     
+    def _select_algorithm(self, *pools):
+        """ Selects an algorithm and iteration direction.
+        
+        Returns:
+            * The algorithm to use (`_eval_internal*`).
+            * The direction in which `next_state()` sees outcomes.
+                1 for ascending and -1 for descending.
+            
+        """
+        has_max_outcomes = any(pool.max_outcomes() is not None for pool in pools)
+        has_min_outcomes = any(pool.min_outcomes() is not None for pool in pools)
+        if has_max_outcomes and has_min_outcomes:
+            raise ValueError('Pools cannot be evaluated if they have both max_outcomes and min_outcomes.')
+        
+        direction = self.direction(*pools)
+        
+        if not direction:
+            if has_max_outcomes:
+                direction = 1
+            elif has_min_outcomes:
+                direction = -1
+            else:
+                num_drop_lowest = max(pool.num_drop_lowest() for pool in pools)
+                num_drop_highest = max(pool.num_drop_highest() for pool in pools)
+                if num_drop_lowest >= num_drop_highest:
+                    direction = 1
+                else:
+                    direction = -1
+        
+        if direction < 0 and has_max_outcomes or direction > 0 and has_min_outcomes:
+            # Forced onto the less-preferred algorithm.
+            return self._eval_internal_iterative, direction
+        else:
+            # Use the preferred algorithm.
+            return self._eval_internal, direction
+    
     def _eval_internal(self, direction, *pools):
-        """ Internal algorithm.
+        """ Internal algorithm for iterating in the more-preferred direction,
+        i.e. giving outcomes to `next_state()` from wide to narrow. 
         
         All intermediate return values are cached in the instance.
         
         Arguments:
             direction: The direction in which to send outcomes to `next_state()`.
             *pools: One or more `DicePool`s to evaluate.
-                Values of `None` indicate that pool has no remaining outcomes to process.
                 This *does* change recursively.
             
         Returns:
@@ -208,27 +228,44 @@ class EvalPool(ABC):
         result = defaultdict(int)
         
         if all(pool.die().is_empty() for pool in pools):
-            result = None
+            result = { None : 1 }
         else:
             outcome, iterators = _pop_pools(direction, pools)
             for p in itertools.product(*iterators):
                 prev_pools, counts, weights = zip(*p)
                 prod_weight = math.prod(weights)
                 prev = self._eval_internal(direction, *prev_pools)
-                if prev is None:
-                    # Base case.
-                    state = self.next_state(None, outcome, *counts)
-                    if state is not None:
-                        result[state] = prod_weight
-                else:
-                    for prev_state, prev_weight in prev.items():
-                        state = self.next_state(prev_state, outcome, *counts)
-                        if state is not None:
-                            result[state] += prev_weight * prod_weight
+                for prev_state, prev_weight in prev.items():
+                    state = self.next_state(prev_state, outcome, *counts)
+                    if state is not hdroller.Reroll:
+                        result[state] += prev_weight * prod_weight
         
         self._cache[cache_key] = result
         return result
     
+    def _eval_internal_iterative(self, direction, *pools):
+        """ Internal algorithm for iterating in the less-preferred direction,
+        i.e. giving outcomes to `next_state()` from narrow to wide.
+        
+        This algorithm does not perform persistent memoization.
+        """
+        dist = defaultdict(int)
+        dist[None, pools] = 1
+        # This is only safe because all pools are guaranteed to pop outcomes at the same rate.
+        while not all(pool.die().is_empty() for pool in pools):
+            next_dist = defaultdict(int)
+            for (prev_state, prev_pools), weight in dist.items():
+                # The direction flip here is the only purpose of this algorithm.
+                outcome, iterators = _pop_pools(-direction, prev_pools)
+                for p in itertools.product(*iterators):
+                    pools, counts, weights = zip(*p)
+                    prod_weight = math.prod(weights)
+                    state = self.next_state(prev_state, outcome, *counts)
+                    if state is not hdroller.Reroll:
+                        next_dist[state, pools] += weight * prod_weight
+            dist = next_dist
+        return { state : weight for (state, _), weight in dist.items() }
+    
 def _pop_pools(side, pools):
     """ Pops a single outcome from the pools.
     
@@ -304,6 +341,10 @@ class SumPool(EvalPool):
             return outcome * count
         else:
             return state + outcome * count
+    
+    def direction(self, *pools):
+        """ This eval doesn't care about direction. """
+        return 0
 
 """ A shared `SumPool` object for caching results. """
 sum_pool = SumPool()
@@ -325,6 +366,10 @@ class FindBestSet(EvalPool):
             return count, outcome
         else:
             return max(state, (count, outcome))
+    
+    def direction(self, *pools):
+        """ This eval doesn't care about direction. """
+        return 0
 
 class FindBestRun(EvalPool):
     """ A `EvalPool` that takes the best run (aka "straight") in a pool.
