@@ -1,7 +1,6 @@
 __docformat__ = 'google'
 
 import hdroller.math
-from hdroller.functools import die_cache
 
 import bisect
 from functools import cached_property
@@ -15,7 +14,7 @@ def Pool(die, num_dice=None, count_dice=None, *, max_outcomes=None, min_outcomes
     
     This should be used in conjunction with `EvalPool` to generate a result.
     
-    You can use `die.pool(args...)` for the same effect as this function.
+    You can use `die.pool(num_dice=None, ...)` for the same effect as this function.
     
     All instances are cached. The members of the actual instance may not match the arguments exactly;
     instead, they may be optimized to values that give the same result as far as `EvalPool` is concerned.
@@ -30,35 +29,20 @@ def Pool(die, num_dice=None, count_dice=None, *, max_outcomes=None, min_outcomes
             they will have 0 count. Zero-weight outcomes will appear with zero weight,
             but can still generate nonzero counts.
         num_dice: An `int` that sets the number of dice in the pool.
-            If no arguments are provided, this defaults to 0.
-        count_dice: Determines which of the **sorted** dice will be counted, and how many times each.
-            
-            `count_dice` can be an `int` or a `slice`, in which case the selected dice are counted once each.
-            For example, `slice(-2, None)` would count the two highest dice.
-            
-            Or `count_dice` can be a sequence of `int`s, one for each die in order.
-            Each die is counted that many times.
-            For example, `[0, 1, 1]` would count the two highest dice out of three.
-            `[0, 0, 2, 0, 0]` would count the middle out of five dice twice.
-            `[-1, 1]` would roll two dice, counting the higher die as a positive and the lower die as a negative.
-            
-            You can also use the `[]` operator to select dice from an existing pool.
-            For example, `pool[-2:]` would also count the two highest dice.
-            This is always an absolute selection on all `num_dice`,
-            not a relative selection on already-selected dice,
-            which would be ambiguous in the presence of multiple or negative counts.
-            
-            The `[]` operator can even change the size of the pool,
-            provided that neither `min_outcomes` nor `max_outcomes` are set.
-            For example, you could create a pool of 4d6 drop lowest using `hdroller.d6.pool()[0, 1, 1, 1]`.
+            If not provided, the number of dice will be inferred from the other arguments.
+            If no arguments are provided at all, this defaults to 0.
+        count_dice: Determines which of the **sorted** dice will be counted, and how many times.
+            Prefer to use the `DicePool`'s `[]` operator after the fact rather than providing an argument here.
+            This operator is an alias for `DicePool.set_count_dice()`.
+            See that method's docstring for details.
         max_outcomes: A sequence of one outcome per die in the pool.
-            That die will be limited to that maximum outcome, with all higher outcomes having 0 weight.
+            That die will be limited to that maximum outcome, with all higher outcomes having 0 count.
             Values cannot be > the `max_outcome` of the fundamental die.
             A pool cannot limit both `min_outcomes` and `max_outcomes`.
             This can be used to efficiently roll a set of mixed standard dice.
             For example, `Pool(hdroller.d12, max_outcomes=[6, 6, 6, 8, 8])` would be a pool of 3d6 and 2d8.
         min_outcomes: A sequence of one outcome per die in the pool.
-            That die will be limited to that minimum outcome, with all lower outcomes having 0 weight.
+            That die will be limited to that minimum outcome, with all lower outcomes having 0 count.
             Values cannot be < the `min_outcome` of the fundamental die.
             A pool cannot limit both `min_outcomes` and `max_outcomes`.
     
@@ -67,25 +51,30 @@ def Pool(die, num_dice=None, count_dice=None, *, max_outcomes=None, min_outcomes
             or if `max_outcome` or `min_outcome` fall outside the range of the die.
     """
     
-    # Compute num_dice.
+    if max_outcomes is not None and min_outcomes is not None:
+        raise ValueError('A pool cannot limit both max_outcomes and min_outcomes.')
     
-    for seq in (count_dice, min_outcomes, max_outcomes):
+    # Compute num_dice and count_dice.
+    
+    for seq in (min_outcomes, max_outcomes):
         if hasattr(seq, '__len__'):
             if num_dice is not None and num_dice != len(seq):
                 raise ValueError('Conflicting values for the number of dice: ' +
-                    f'num_dice={num_dice}, count_dice={count_dice}, min_outcomes={min_outcomes}, max_outcomes={max_outcomes}')
+                    f'num_dice={num_dice}, min_outcomes={min_outcomes}, max_outcomes={max_outcomes}')
             else:
                 num_dice = len(seq)
-        
+    
+    convert_to_die = isinstance(count_dice, int)
+    
     if num_dice is None:
-        num_dice = 0
-    
-    # Compute count_dice.
-    
-    if count_dice is None:
-        count_dice = (1,) * num_dice
+        # Default to zero dice, unless count_dice has something to say.
+        count_dice = count_dice_tuple(0, count_dice)
+        num_dice = len(count_dice)
     else:
-        count_dice = _compute_count_dice(num_dice, count_dice)
+        # Must match num_dice.
+        count_dice = count_dice_tuple(num_dice, count_dice)
+        if len(count_dice) != num_dice:
+            raise ValueError(f'The length of count_dice={count_dice} conflicts with num_dice={num_dice}.')
     
     # Put max/min outcomes into standard form.
     # This is either a sorted tuple, or `None` if there is no (effective) limit to the die size on that side.
@@ -114,42 +103,95 @@ def Pool(die, num_dice=None, count_dice=None, *, max_outcomes=None, min_outcomes
                 # In this case, the min_outcomes don't actually do anything.
                 min_outcomes = None
     
-    if max_outcomes is not None and min_outcomes is not None:
-        raise ValueError('A pool cannot limit both max_outcomes and min_outcomes.')
-    
-    return _pool_cached_unchecked(die, count_dice, max_outcomes, min_outcomes)
+    result = _pool_cached_unchecked(die, count_dice, max_outcomes, min_outcomes)
+    if convert_to_die:
+        return result.eval(lambda state, outcome, count: outcome if count else state)
+    else:
+        return result
 
-def _compute_count_dice(num_dice, count_dice):
-    """ Returns a tuple specifying count_dice.
+def count_dice_tuple(num_dice, count_dice):
+    """ Expresses `count_dice` as a tuple.
     
-    If `count_dice` is already a sequence, this does not check its length against `num_dice`.
+    See `DicePool.set_count_dice()` for details.
+    
+    Args:
+        `num_dice`: An `int` specifying the number of dice.
+        `count_dice`: Raw specification for how the dice are to be counted.
+    Raises:
+        `ValueError` if:
+            * More than one `Ellipsis` is used.
+            * An `Ellipsis` is used in the center with too few `num_dice`.
     """
-    if isinstance(count_dice, int):
+    if count_dice is None:
+        return (1,) * num_dice
+    elif isinstance(count_dice, int):
         result = [0] * num_dice
         result[count_dice] = 1
         return tuple(result)
     elif isinstance(count_dice, slice):
+        if count_dice.step is not None:
+            # "Step" is not useful here, so we repurpose it to set the number of dice.
+            num_dice = count_dice.step
+            count_dice = slice(count_dice.start, count_dice.stop)
         result = [0] * num_dice
         result[count_dice] = [1] * len(result[count_dice])
         return tuple(result)
     else:
-        if not all(isinstance(x, int) for x in count_dice):
-            raise TypeError('count_dice must be a sequence of ints.')
-        return tuple(count_dice)
+        split = None
+        for i, x in enumerate(count_dice):
+            if x is Ellipsis:
+                if split is None:
+                    split = i
+                else:
+                    raise ValueError('Cannot use more than one Ellipsis (...) for count_dice.')
 
-@die_cache
-def _pool_cached_unchecked(die, count_dice, max_outcomes, min_outcomes):
+        if split is None:
+            return tuple(count_dice)
+        
+        extra_dice = num_dice - len(count_dice) + 1
+        
+        if split == 0:
+            # Ellipsis on left.
+            count_dice = count_dice[1:]
+            if extra_dice < 0:
+                return tuple(count_dice[-extra_dice:])
+            else:
+                return (0,) * extra_dice + tuple(count_dice)
+        elif split == len(count_dice) - 1:
+            # Ellipsis on right.
+            count_dice = count_dice[:-1]
+            if extra_dice < 0:
+                return tuple(count_dice[:extra_dice])
+            else:
+                return tuple(count_dice) + (0,) * extra_dice
+        else:
+            # Ellipsis in center.
+            if extra_dice < 0:
+                raise ValueError(f'count_dice={count_dice} with Ellipsis (...) in center has too many elements for num_dice={num_dice}.')
+            return tuple(count_dice[:split]) + (0,) * extra_dice + tuple(count_dice[split+1:])
+
+_pool_cache = {}
+
+def _pool_cached_unchecked(die, count_dice, max_outcomes=None, min_outcomes=None):
     """ Cached, unchecked constructor for dice pools.
     
     This should not be used directly. Use the `Pool()` factory function instead.
     """
-    return DicePool(die, count_dice, max_outcomes=max_outcomes, min_outcomes=min_outcomes)
+    key = (die.key_tuple(), count_dice, max_outcomes, min_outcomes)
+    if key in _pool_cache:
+        return _pool_cache[key]
+    else:
+        result = DicePool(die, count_dice, max_outcomes=max_outcomes, min_outcomes=min_outcomes)
+    _pool_cache[key] = result
+    return result
 
 def standard_pool(*die_sizes, count_dice=None):
     """ Creates a pool of standard dice.
     
     For example, `standard_pool(8, 8, 6, 6, 6)` would be a pool of 2 d8s and 3 d6s.
     
+    If no die sizes are given, the pool will consist of zero d1s.
+    
     Args:
         *die_sizes: The size of each die in the pool.
         count_dice: Which dice will be counted, as `Pool()`.
@@ -157,6 +199,8 @@ def standard_pool(*die_sizes, count_dice=None):
             For example, `standard_pool(8, 8, 6, 6, 6)[-2:]` would keep the highest two dice
             of 2 d8s and 3 d6s.
     """
+    if len(die_sizes) == 0:
+        return Pool(hdroller.d1, num_dice=0)
     return Pool(hdroller.d(max(die_sizes)), count_dice=count_dice, max_outcomes=die_sizes)
 
 class DicePool():
@@ -184,6 +228,77 @@ class DicePool():
         """ A tuple indicating how many times each of the dice, sorted from lowest to highest, counts. """
         return self._count_dice
     
+    def num_dice(self):
+        """ The number of dice in this pool (before dropping or counting multiple times). """
+        return len(self._count_dice)
+    
+    def set_count_dice(self, count_dice):
+        """ Returns a pool with the selected dice counted.
+        
+        You can use `pool[count_dice]` for the same effect as this method.
+        
+        The dice are sorted in ascending order for this purpose,
+        regardless of which order the outcomes are evaluated in.
+        
+        This is always an absolute selection on all `num_dice`,
+        not a relative selection on already-selected dice,
+        which would be ambiguous in the presence of multiple or negative counts.
+        
+        Args:
+            `None`: All dice will be counted once.
+            An `int`. This will count only the die at the specified index (once).
+                In this case, the result will be a die, not a pool.
+            A `slice`. The selected dice are counted once each.
+                If provided, the third argument resizes the pool,
+                rather than being a step,
+                but only if the pool does not have `max_outcomes` or `min_outcomes`.
+            A sequence of one `int`s for each die.
+                Each die is counted that many times, which could be multiple or negative times.
+                This may resize the pool, but only if the pool does not have `max_outcomes` or `min_outcomes`.
+                Up to one `Ellipsis` (`...`) may be used.
+                If one is used, `count_dice` will be padded with zero counts if shorter than `num_dice`,
+                or trimmed if longer.
+        
+        Raises:
+            ValueError if:
+                * `count_dice` would change the size of a pool with `max_outcomes` or `min_outcomes`.
+                * More than one `Ellipsis` is used.
+                * `Ellipsis` is used in the center with too many elements for `num_dice`.
+        
+        For example, here are some ways of selecting the two highest dice out of 5:
+        
+        * `pool[3:5]`
+        * `pool[3:]`
+        * `pool[-2:]`
+        * `pool[..., 1, 1]`
+        
+        These will also select the two highest dice out of 5, and will also resize the pool to 5 dice first:
+        
+        * `pool[3::5]`
+        * `pool[3:5:5]`
+        * `pool[-2::5]`
+        * `pool[0, 0, 0, 1, 1]`
+        
+        These will count the highest as a positive and the lowest as a negative:
+        
+        * `pool[-1, 0, 0, 0, 1]`
+        * `pool[-1, ..., 1]`
+        """
+        convert_to_die = isinstance(count_dice, int)
+        count_dice = count_dice_tuple(self.num_dice(), count_dice)
+        if len(count_dice) != self.num_dice():
+            if self.max_outcomes() is not None:
+                raise ValueError('Cannot change the size of a pool with max_outcomes.')
+            if self.min_outcomes() is not None:
+                raise ValueError('Cannot change the size of a pool with min_outcomes.')
+        result = _pool_cached_unchecked(self.die(), count_dice=count_dice, max_outcomes=self.max_outcomes(), min_outcomes=self.min_outcomes())
+        if convert_to_die:
+            return result.eval(lambda state, outcome, count: outcome if count else state)
+        else:
+            return result
+    
+    __getitem__ = set_count_dice
+    
     @cached_property
     def _num_drop_lowest(self):
         for i, count in enumerate(self.count_dice()):
@@ -205,38 +320,6 @@ class DicePool():
     def num_drop_highest(self):
         """ How many elements of count_dice on the high side have a false truth value. """
         return self._num_drop_highest
-        
-    def __getitem__(self, count_dice):
-        """ Returns a pool with the selected dice counted, as the `count_dice` argument to `Pool()`.
-        
-            Determines which of the **sorted** dice will be counted, and how many times.
-            The dice are sorted in ascending order for this purpose,
-            regardless of which order the outcomes are evaluated in.
-            
-            This can be an `int` or a `slice`, in which case the selected dice are counted once each.
-            For example, `slice(-2, None)` would count the two highest dice.
-            
-            Or this can be a sequence of `int`s, one for each die in order.
-            Each die is counted that many times.
-            For example, `[-2:]` would also count the two highest dice.
-            `[0, 0, 2, 0, 0]` would count the middle out of five dice twice.
-            `[-1, 1]` would roll two dice, counting the higher die as a positive and the lower die as a negative.
-            This can change the size of the pool, but only if neither `min_outcomes` or `max_outcomes` are set.
-            
-            This is always an absolute selection on all `num_dice`,
-            not a relative selection on already-selected dice,
-            which would be ambiguous in the presence of multiple or negative counts.
-        """
-        count_dice = _compute_count_dice(self.num_dice(), count_dice)
-        if len(count_dice) != self.num_dice():
-            if self.max_outcomes() is not None:
-                raise ValueError('The [] operator cannot change the size of a pool with max_outcomes.')
-            if self.min_outcomes() is not None:
-                raise ValueError('The [] operator cannot change the size of a pool with min_outcomes.')
-        return Pool(self.die(), count_dice=count_dice, max_outcomes=self.max_outcomes(), min_outcomes=self.min_outcomes())
-    
-    def num_dice(self):
-        return len(self._count_dice)
     
     def max_outcomes(self, always_tuple=False):
         """ A tuple of sorted max outcomes, one for each die in the pool. 
@@ -278,7 +361,7 @@ class DicePool():
         
         if popped_die.is_empty():
             # This is the last outcome. All dice must roll this outcome.
-            pool = Pool(popped_die, num_dice=0)
+            pool = _pool_cached_unchecked(popped_die, count_dice=())
             remaining_count = sum(self.count_dice())
             weight = single_weight ** num_possible_dice
             yield pool, remaining_count, weight
@@ -292,14 +375,14 @@ class DicePool():
         comb_row = hdroller.math.comb_row(num_possible_dice, single_weight)
         end_counted = self.num_dice() - self.num_drop_lowest()
         for weight in comb_row[:min(num_possible_dice, end_counted)]:
-            pool = Pool(popped_die, count_dice=popped_count_dice, max_outcomes=popped_max_outcomes)
+            pool = _pool_cached_unchecked(popped_die, count_dice=popped_count_dice, max_outcomes=popped_max_outcomes)
             yield pool, count, weight
             count += popped_count_dice[-1]
             popped_max_outcomes = popped_max_outcomes[:-1]
             popped_count_dice = popped_count_dice[:-1]
         
         if end_counted > num_possible_dice:
-            pool = Pool(popped_die, count_dice=popped_count_dice, max_outcomes=popped_max_outcomes)
+            pool = _pool_cached_unchecked(popped_die, count_dice=popped_count_dice, max_outcomes=popped_max_outcomes)
             yield pool, count, comb_row[-1]
         else:
             # In this case, we ran out of counted dice before running out of dice that could roll the outcome.
@@ -309,7 +392,7 @@ class DicePool():
                 skip_weight *= popped_die.total_weight()
                 skip_weight += weight
             skip_weight *= math.prod(popped_die.weight_le(max_outcome) for max_outcome in max_outcomes[:num_unused_dice])
-            pool = Pool(popped_die, 0)
+            pool = _pool_cached_unchecked(popped_die, count_dice=())
             yield pool, count, skip_weight
     
     def _iter_pop_min(self):
@@ -333,7 +416,7 @@ class DicePool():
         
         if popped_die.is_empty():
             # This is the last outcome. All dice must roll this outcome.
-            pool = Pool(popped_die, num_dice=0)
+            pool = _pool_cached_unchecked(popped_die, count_dice=())
             remaining_count = sum(self.count_dice())
             weight = single_weight ** num_possible_dice
             yield pool, remaining_count, weight
@@ -347,14 +430,14 @@ class DicePool():
         comb_row = hdroller.math.comb_row(num_possible_dice, single_weight)
         end_counted = self.num_dice() - self.num_drop_highest()
         for weight in comb_row[:min(num_possible_dice, end_counted)]:
-            pool = Pool(popped_die, count_dice=popped_count_dice, min_outcomes=popped_min_outcomes)
+            pool = _pool_cached_unchecked(popped_die, count_dice=popped_count_dice, min_outcomes=popped_min_outcomes)
             yield pool, count, weight
             count += popped_count_dice[0]
             popped_min_outcomes = popped_min_outcomes[1:]
             popped_count_dice = popped_count_dice[1:]
         
         if end_counted > num_possible_dice:
-            pool = Pool(popped_die, count_dice=popped_count_dice, min_outcomes=popped_min_outcomes)
+            pool = _pool_cached_unchecked(popped_die, count_dice=popped_count_dice, min_outcomes=popped_min_outcomes)
             yield pool, count, comb_row[-1]
         else:
             # In this case, we ran out of counted dice before running out of dice that could roll the outcome.
@@ -364,7 +447,7 @@ class DicePool():
                 skip_weight *= popped_die.total_weight()
                 skip_weight += weight
             skip_weight *= math.prod(popped_die.weight_ge(min_outcome) for min_outcome in min_outcomes[num_possible_dice:])
-            pool = Pool(popped_die, 0)
+            pool = _pool_cached_unchecked(popped_die, count_dice=())
             yield pool, count, skip_weight
     
     @cached_property
